@@ -14,10 +14,46 @@
 from abc import ABC
 import numpy as np
 import pandas
+import warnings
 
 from modin.error_message import ErrorMessage
 from modin.data_management.utils import compute_chunksize
+from modin.config import NPartitions, ProgressBar, BenchmarkMode
+
 from pandas.api.types import union_categoricals
+import os
+
+
+def wait_computations_if_benchmark_mode(func):
+    """
+    Synchronize performing a function in benchmark mode.
+
+    Parameters
+    ----------
+    func: callable
+        A function that should be performed in syncronous mode.
+
+    Note
+    ----
+        `func` should return numpy array with partitions.
+    """
+    if BenchmarkMode.get():
+
+        def wait(*args, **kwargs):
+            """Wait for computation results."""
+            result = func(*args, **kwargs)
+            if isinstance(result, tuple):
+                partitions = result[0]
+            else:
+                partitions = result
+            # need to go through all the values of the map iterator
+            # since `wait` does not return anything, we need to explicitly add
+            # the return `True` value from the lambda
+            all(map(lambda partition: partition.wait() or True, partitions.flatten()))
+            return result
+
+        return wait
+    return func
 
 
 class BaseFrameManager(ABC):
@@ -140,6 +176,7 @@ class BaseFrameManager(ABC):
         )
 
     @classmethod
+    @wait_computations_if_benchmark_mode
     def broadcast_apply_select_indices(
         cls,
         axis,
@@ -210,6 +247,7 @@ class BaseFrameManager(ABC):
         return new_partitions
 
     @classmethod
+    @wait_computations_if_benchmark_mode
     def broadcast_apply(cls, axis, apply_func, left, right, other_name="r"):
         """Broadcast the right partitions to left and apply a function.
 
@@ -257,6 +295,7 @@ class BaseFrameManager(ABC):
         return new_partitions
 
     @classmethod
+    @wait_computations_if_benchmark_mode
     def broadcast_axis_partitions(
         cls,
         axis,
@@ -299,7 +338,7 @@ class BaseFrameManager(ABC):
         elif lengths:
             num_splits = len(lengths)
         else:
-            num_splits = cls._compute_num_partitions()
+            num_splits = NPartitions.get()
         preprocessed_map_func = cls.preprocess_func(apply_func)
         left_partitions = cls.axis_partition(left, axis)
         right_partitions = None if right is None else cls.axis_partition(right, axis)
@@ -334,6 +373,7 @@ class BaseFrameManager(ABC):
         return result_blocks.T if not axis else result_blocks
 
     @classmethod
+    @wait_computations_if_benchmark_mode
     def map_partitions(cls, partitions, map_func):
         """Apply `map_func` to every partition.
 
@@ -355,6 +395,7 @@ class BaseFrameManager(ABC):
         )
 
     @classmethod
+    @wait_computations_if_benchmark_mode
     def lazy_map_partitions(cls, partitions, map_func):
         """
         Apply `map_func` to every partition lazily.
@@ -426,6 +467,7 @@ class BaseFrameManager(ABC):
         )
 
     @classmethod
+    @wait_computations_if_benchmark_mode
     def simple_shuffle(cls, axis, partitions, map_func, lengths):
         """
         Shuffle data using `lengths` via `map_func`.
@@ -565,18 +607,56 @@ class BaseFrameManager(ABC):
         )
 
     @classmethod
+    @wait_computations_if_benchmark_mode
     def from_pandas(cls, df, return_dims=False):
         """Return the partitions from Pandas DataFrame."""
-        num_splits = cls._compute_num_partitions()
+
+        def update_bar(pbar, f):
+            if ProgressBar.get():
+                pbar.update(1)
+            return f
+
+        num_splits = NPartitions.get()
         put_func = cls._partition_class.put
         row_chunksize, col_chunksize = compute_chunksize(df, num_splits)
+
+        bar_format = (
+            "{l_bar}{bar}{r_bar}"
+            if os.environ.get("DEBUG_PROGRESS_BAR", "False") == "True"
+            else "{desc}: {percentage:3.0f}%{bar} Elapsed time: {elapsed}, estimated remaining time: {remaining}"
+        )
+        if ProgressBar.get():
+            with warnings.catch_warnings():
+                warnings.simplefilter("ignore")
+                try:
+                    from tqdm.autonotebook import tqdm as tqdm_notebook
+                except ImportError:
+                    raise ImportError("Please pip install tqdm to use the progress bar")
+
+            rows = max(1, round(len(df) / row_chunksize))
+            cols = max(1, round(len(df.columns) / col_chunksize))
+            update_count = rows * cols
+            pbar = tqdm_notebook(
+                total=round(update_count),
+                desc="Distributing Dataframe",
+                bar_format=bar_format,
+            )
+        else:
+            pbar = None
         parts = [
             [
-                put_func(df.iloc[i : i + row_chunksize, j : j + col_chunksize].copy())
+                update_bar(
+                    pbar,
+                    put_func(
+                        df.iloc[i : i + row_chunksize, j : j + col_chunksize].copy()
+                    ),
+                )
                 for j in range(0, len(df.columns), col_chunksize)
             ]
             for i in range(0, len(df), row_chunksize)
         ]
+        if ProgressBar.get():
+            pbar.close()
         if not return_dims:
             return np.array(parts)
         else:
@@ -632,18 +712,6 @@ class BaseFrameManager(ABC):
         # TODO FIX INFORMATION LEAK!!!!1!!1!!
         return new_idx[0].append(new_idx[1:]) if len(new_idx) else new_idx
 
-    @classmethod
-    def _compute_num_partitions(cls):
-        """Retrieve the default number of partitions currently. Will estimate the optimal no. of partitions in future.
-
-        Returns
-        -------
-            Number of partitions.
-        """
-        from modin.pandas import DEFAULT_NPARTITIONS
-
-        return DEFAULT_NPARTITIONS
-
     @classmethod
     def _apply_func_to_list_of_partitions_broadcast(
         cls, func, partitions, other, **kwargs
@@ -672,6 +740,7 @@ class BaseFrameManager(ABC):
         return [obj.apply(preprocessed_func, **kwargs) for obj in partitions]
 
     @classmethod
+    @wait_computations_if_benchmark_mode
     def apply_func_to_select_indices(
         cls, axis, partitions, func, indices, keep_remaining=False
     ):
@@ -774,6 +843,7 @@ class BaseFrameManager(ABC):
         return result.T if not axis else result
 
     @classmethod
+    @wait_computations_if_benchmark_mode
     def apply_func_to_select_indices_along_full_axis(
         cls, axis, partitions, func, indices, keep_remaining=False
     ):
@@ -883,6 +953,7 @@ class BaseFrameManager(ABC):
         return result.T if not axis else result
 
     @classmethod
+    @wait_computations_if_benchmark_mode
     def apply_func_to_indices_both_axis(
         cls,
         partitions,
@@ -929,6 +1000,7 @@ class BaseFrameManager(ABC):
         return partition_copy
 
     @classmethod
+    @wait_computations_if_benchmark_mode
     def binary_operation(cls, axis, left, func, right):
         """
         Apply a function that requires two BasePandasFrame objects.
@@ -960,7 +1032,7 @@ class BaseFrameManager(ABC):
             [
                 left_partitions[i].apply(
                     func,
-                    num_splits=cls._compute_num_partitions(),
+                    num_splits=NPartitions.get(),
                     other_axis_partition=right_partitions[i],
                 )
                 for i in range(len(left_partitions))
