@@ -43,6 +43,9 @@ class OmnisciOnRayFramePartition(BaseFramePartition):
             return self.arrow_table
         return ray.get(self.oid)
 
+    def wait(self):
+        ray.wait([self.oid])
+
     @classmethod
     def put(cls, obj):
         return OmnisciOnRayFramePartition(
