@@ -36,12 +36,13 @@ import sys
 from typing import Union, Optional
 import warnings
 
-from modin.utils import _inherit_docstrings, to_pandas
-from modin.config import IsExperimental
+from modin.utils import _inherit_docstrings, to_pandas, Engine
+from modin.config import IsExperimental, PersistentPickle
 from .base import BasePandasDataset, _ATTRS_NO_LOOKUP
 from .iterator import PartitionIterator
 from .utils import from_pandas, is_scalar
 from .accessor import CachedAccessor, SparseAccessor
+from . import _update_engine
 
 
 @_inherit_docstrings(pandas.Series, excluded=[pandas.Series.__init__])
@@ -77,6 +78,7 @@ class Series(BasePandasDataset):
         query_compiler: query_compiler
             A query compiler object to create the Series from.
         """
+        Engine.subscribe(_update_engine)
         if isinstance(data, type(self)):
             query_compiler = data._query_compiler.copy()
             if index is not None:
@@ -140,9 +142,17 @@ class Series(BasePandasDataset):
         return self.add(left)
 
     def __and__(self, other):
+        if isinstance(other, (list, np.ndarray, pandas.Series)):
+            return self._default_to_pandas(pandas.Series.__and__, other)
         new_self, new_other = self._prepare_inter_op(other)
         return super(Series, new_self).__and__(new_other)
 
+    def __rand__(self, other):
+        if isinstance(other, (list, np.ndarray, pandas.Series)):
+            return self._default_to_pandas(pandas.Series.__rand__, other)
+        new_self, new_other = self._prepare_inter_op(other)
+        return super(Series, new_self).__rand__(new_other)
+
     def __array__(self, dtype=None):
         return super(Series, self).__array__(dtype).flatten()
 
@@ -167,12 +177,6 @@ class Series(BasePandasDataset):
             raise KeyError(key)
         self.drop(labels=key, inplace=True)
 
-    def __div__(self, right):
-        return self.div(right)
-
-    def __rdiv__(self, left):
-        return self.rdiv(left)
-
     def __divmod__(self, right):
         return self.divmod(right)
 
@@ -215,9 +219,29 @@ class Series(BasePandasDataset):
         return self.rmul(left)
 
     def __or__(self, other):
+        if isinstance(other, (list, np.ndarray, pandas.Series)):
+            return self._default_to_pandas(pandas.Series.__or__, other)
         new_self, new_other = self._prepare_inter_op(other)
         return super(Series, new_self).__or__(new_other)
 
+    def __ror__(self, other):
+        if isinstance(other, (list, np.ndarray, pandas.Series)):
+            return self._default_to_pandas(pandas.Series.__ror__, other)
+        new_self, new_other = self._prepare_inter_op(other)
+        return super(Series, new_self).__ror__(new_other)
+
+    def __xor__(self, other):
+        if isinstance(other, (list, np.ndarray, pandas.Series)):
+            return self._default_to_pandas(pandas.Series.__xor__, other)
+        new_self, new_other = self._prepare_inter_op(other)
+        return super(Series, new_self).__xor__(new_other)
+
+    def __rxor__(self, other):
+        if isinstance(other, (list, np.ndarray, pandas.Series)):
+            return self._default_to_pandas(pandas.Series.__rxor__, other)
+        new_self, new_other = self._prepare_inter_op(other)
+        return super(Series, new_self).__rxor__(new_other)
+
     def __pow__(self, right):
         return self.pow(right)
 
@@ -261,11 +285,10 @@ class Series(BasePandasDataset):
         )
 
     def __setitem__(self, key, value):
-        if key not in self.keys():
-            raise KeyError(key)
-        self._create_or_update_from_compiler(
-            self._query_compiler.setitem(1, key, value), inplace=True
-        )
+        if isinstance(key, slice):
+            self._setitem_slice(key, value)
+        else:
+            self.loc[key] = value
 
     def __sub__(self, right):
         return self.sub(right)
@@ -289,10 +312,6 @@ class Series(BasePandasDataset):
     def values(self):
         return super(Series, self).to_numpy().flatten()
 
-    def __xor__(self, other):
-        new_self, new_other = self._prepare_inter_op(other)
-        return super(Series, new_self).__xor__(new_other)
-
     def add(self, other, level=None, fill_value=None, axis=0):
         new_self, new_other = self._prepare_inter_op(other)
         return super(Series, new_self).add(
@@ -480,13 +499,21 @@ class Series(BasePandasDataset):
         keep_shape: bool = False,
         keep_equal: bool = False,
     ):
-        return self._default_to_pandas(
-            pandas.Series.compare,
-            other=other,
+        if not isinstance(other, Series):
+            raise TypeError(f"Cannot compare Series to {type(other)}")
+        result = self.to_frame().compare(
+            other.to_frame(),
             align_axis=align_axis,
             keep_shape=keep_shape,
             keep_equal=keep_equal,
         )
+        if align_axis == "columns" or align_axis == 1:
+            # Pandas.DataFrame.Compare returns a dataframe with a multidimensional index object as the
+            # columns so we have to change column object back.
+            result.columns = pandas.Index(["self", "other"])
+        else:
+            result = result.squeeze().rename(None)
+        return result
 
     def corr(self, other, method="pearson", min_periods=None):
         if method == "pearson":
@@ -878,6 +905,11 @@ class Series(BasePandasDataset):
                 )
             )
 
+    def shift(self, periods=1, freq=None, axis=0, fill_value=None):
+        return super(type(self), self).shift(
+            periods=periods, freq=freq, axis=axis, fill_value=fill_value
+        )
+
     def unstack(self, level=-1, fill_value=None):
         from .dataframe import DataFrame
 
@@ -933,22 +965,22 @@ class Series(BasePandasDataset):
         **kwargs,
     ):
         axis = self._get_axis_number(axis)
+        if level is not None:
+            if (
+                not self._query_compiler.has_multiindex(axis=axis)
+                and level > 0
+                or level < -1
+                and level != self.index.name
+            ):
+                raise ValueError("level > 0 or level < -1 only valid with MultiIndex")
+            return self.groupby(level=level, axis=axis, sort=False).prod(
+                numeric_only=numeric_only, min_count=min_count, **kwargs
+            )
         new_index = self.columns if axis else self.index
         if min_count > len(new_index):
             return np.nan
 
         data = self._validate_dtypes_sum_prod_mean(axis, numeric_only, ignore_axis=True)
-        if level is not None:
-            return data.__constructor__(
-                query_compiler=data._query_compiler.prod_min_count(
-                    axis=axis,
-                    skipna=skipna,
-                    level=level,
-                    numeric_only=numeric_only,
-                    min_count=min_count,
-                    **kwargs,
-                )
-            )
         if min_count > 1:
             return data._reduce_dimension(
                 data._query_compiler.prod_min_count(
@@ -1217,6 +1249,18 @@ class Series(BasePandasDataset):
         **kwargs,
     ):
         axis = self._get_axis_number(axis)
+        if level is not None:
+            if (
+                not self._query_compiler.has_multiindex(axis=axis)
+                and level > 0
+                or level < -1
+                and level != self.index.name
+            ):
+                raise ValueError("level > 0 or level < -1 only valid with MultiIndex")
+            return self.groupby(level=level, axis=axis, sort=False).sum(
+                numeric_only=numeric_only, min_count=min_count, **kwargs
+            )
+
         new_index = self.columns if axis else self.index
         if min_count > len(new_index):
             return np.nan
@@ -1224,17 +1268,6 @@ class Series(BasePandasDataset):
         data = self._validate_dtypes_sum_prod_mean(
             axis, numeric_only, ignore_axis=False
         )
-        if level is not None:
-            return data.__constructor__(
-                query_compiler=data._query_compiler.sum_min_count(
-                    axis=axis,
-                    skipna=skipna,
-                    level=level,
-                    numeric_only=numeric_only,
-                    min_count=min_count,
-                    **kwargs,
-                )
-            )
         if min_count > 1:
             return data._reduce_dimension(
                 data._query_compiler.sum_min_count(
@@ -1445,7 +1478,7 @@ class Series(BasePandasDataset):
 
     @property
     def is_monotonic(self):
-        return self._reduce_dimension(self._query_compiler.is_monotonic())
+        return self._reduce_dimension(self._query_compiler.is_monotonic_increasing())
 
     is_monotonic_increasing = is_monotonic
 
@@ -1701,6 +1734,23 @@ class Series(BasePandasDataset):
             return self._reduce_dimension(result)
         return self.__constructor__(query_compiler=result)
 
+    # Persistance support methods - BEGIN
+    @classmethod
+    def _inflate_light(cls, query_compiler, name):
+        return cls(query_compiler=query_compiler, name=name)
+
+    @classmethod
+    def _inflate_full(cls, pandas_series):
+        return cls(data=pandas_series)
+
+    def __reduce__(self):
+        self._query_compiler.finalize()
+        if PersistentPickle.get():
+            return self._inflate_full, (self._to_pandas(),)
+        return self._inflate_light, (self._query_compiler, self.name)
+
+    # Persistance support methods - END
+
 
 if IsExperimental.get():
     from modin.experimental.cloud.meta_magic import make_wrapped_class
